#include "adaline.h"
# include <stdlib.h>
# include <math.h>
Adaline::Adaline()
{
    n=0.1;
    maxepochs=100;
}

void    Adaline::setTrainSet(Dataset *t)
{
    trainSet=t;
    weight.resize(trainSet->getdimension()+1);
    int i;
    for(i=0;i<weight.size();i++)
    {
        weight[i]=0.1 * (rand() *1.0/RAND_MAX);
    }
    for(i=0;i<trainSet->getpatterns();i++)
    {
        double y=trainSet->gety(i);
        if(classVector.indexOf(y)==-1)
            classVector<<y;
    }
}

void    Adaline::setTestSet(Dataset *t)
{
    testSet=t;
}

void    Adaline::setN(double mn)
{
    if(mn>0 && mn<1) n=mn;
}

double  Adaline::getN() const
{
    return n;
}

void    Adaline::setMaxEpochs(int e)
{
    if(e>0) maxepochs=e;
}

int     Adaline::getMaxEpochs() const
{
    return maxepochs;
}

void    Adaline::train()
{
    int k;
    for(k=1;k<=maxepochs;k++)
    {
        int i;
        for(i=0;i<trainSet->getpatterns();i++)
        {
            Data pattern=trainSet->getpoint(i);
            pattern<<1;
            double u=getOutput(pattern);
            int j;
            for(j=0;j<weight.size();j++)
                weight[j]=weight[j]+n*(trainSet->gety(i)-u)*pattern[j];
        }
        double e=getTrainError();
        if(e<1e-4) break;
    }
}

double Adaline::getTrainError()
{
    double sum=0.0;
    int i;
    for(i=0;i<trainSet->getpatterns();i++)
    {
        Data pattern=trainSet->getpoint(i);
        pattern<<1;
        double u=getOutput(pattern);
        double y=trainSet->gety(i);
        sum+=(u-y)*(u-y);
    }
    return sum/trainSet->getpatterns();
}

double  Adaline::getTestError()
{
    double sum=0.0;
    int i;
    for(i=0;i<testSet->getpatterns();i++)
    {
        Data pattern=testSet->getpoint(i);
        pattern<<1;
        double u=getOutput(pattern);
        double y=testSet->gety(i);
        sum+=(u-y)*(u-y);
    }
    return sum/testSet->getpatterns();
}

double  Adaline::getClassError()
{
    int missed=0;
    int i;
    for(i=0;i<testSet->getpatterns();i++)
    {
        Data pattern=testSet->getpoint(i);
        pattern<<1;
        double d=getClass(pattern);
        double y=testSet->gety(i);
        if(fabs(d-y)>1e-5) missed++;
    }
    return missed * 100.0/testSet->getpatterns();
}

double  Adaline::getOutput(Data pattern)
{
    double f=product(pattern,weight);
    return f;
}

double  Adaline::getClass(Data pattern)
{
    double f=product(pattern,weight);
    int minClass=-1;
    double minDist=1e+10;
    int i;
    for(i=0;i<classVector.size();i++)
    {
        double dist=fabs(classVector[i]-f);
        if(dist<minDist)
        {
            minClass=i;
            minDist=dist;
        }
    }
    return classVector[minClass];
}


double  Adaline::product(Data x, Data y)
{
    int i;
    double s=0.0;
    for(i=0;i<x.size();i++)
        s+=x[i]*y[i];
    return s;
}
